pub fn get_primes(max_nr: usize) -> Vec<u64> {
    let mut sieve = vec![true; max_nr + 1];

    let mut primes = Vec::with_capacity(1_000_000);

    sieve[0] = false;
    sieve[1] = false;

    let mut nr = 2;
    primes.push(2);

    loop {
        check_off_multiples_of_nr(nr, &mut sieve);

        if find_next_prime(&mut nr, &mut sieve).is_none() {
            break;
        }

        primes.push(nr);
    }

    primes
}

fn find_next_prime(nr: &mut u64, sieve: &mut [bool]) -> Option<u64> {
    let start_idx = usize::try_from(*nr).ok()? + 1;

    if let Some((idx, _)) = sieve
        .iter()
        .enumerate()
        .skip(start_idx)
        .find(|&(_, &is_prime)| is_prime)
    {
        *nr = idx as u64;
        return Some(*nr);
    }

    None
}

fn check_off_multiples_of_nr(nr: u64, sieve: &mut [bool]) {

    let nr = usize::try_from(nr).unwrap();
    
    sieve
        .iter_mut()
        .skip(nr.checked_mul(2).unwrap())
        .step_by(nr)
        .for_each(|nr| *nr = false);
}
